/** ADAMEm: Coleco ADAM emulator ********************************************/
/**                                                                        **/
/**                               tapeconv.c                               **/
/**                                                                        **/
/** This program converts disk images made with tapeconv.dsk to tape       **/
/** images                                                                 **/
/**                                                                        **/
/** Copyright (C) Marcel de Kogel 1997                                     **/
/**     You are not allowed to distribute this software commercially       **/
/**     Please, notify me, if you make any changes to this file            **/
/****************************************************************************/

#include <stdio.h>

static char buffer[256*1024];
static int interleave[]= { 0,5,2,7,4,1,6,3 };

static int WriteSector (FILE *f,int n)
{
 int track,sector;
 track=n/8;
 n%=8;
 for (sector=0;;++sector)
  if (interleave[sector]==n) break;
 return (fwrite(buffer+track*4096+sector*512,1,512,f)==512);
}

int main (int argc,char *argv[])
{
 FILE *f;
 char *d1=NULL,*d2=NULL,*t=NULL;
 int misparm=0,i,n;
 printf ("tapeconv: ADAM tape image converter\n"
         "Copyright (C) Marcel de Kogel 1997\n");
 for (n=1,i=0;n<argc;n++)
 {
  if(*argv[n]!='-')
   switch(i++)
   {
    case 0:  d1=argv[n];
             break;
    case 1:  d2=argv[n];
             break;
    case 2:  t=argv[n];
             break;
    default: misparm=1;
             break;
   }
  else
  {
   /* No options */
   misparm=1;
  }
 }
 if (misparm || !d1 || !d2 || !t)
 {
  printf ("Usage: tapeconv <source 1> <source 2> <tape image>\n");
  return 1;
 }
 f=fopen (d1,"rb");
 if (!f)
 {
  printf ("Cannot open %s\n",d1);
  return 2;
 }
 if (fread(buffer,1024,128,f)!=128)
 {
  printf ("Error reading %s\n",d1);
  return 3;
 }
 fclose (f);
 f=fopen (d2,"rb");
 if (!f)
 {
  printf ("Cannot open %s\n",d2);
  fclose (f);
  return 2;
 }
 if (fread(buffer+128*1024,1024,128,f)!=128)
 {
  printf ("Error reading %s\n",d2);
  fclose (f);
  return 3;
 }
 fclose (f);
 f=fopen (t,"wb");
 if (!f)
 {
  printf ("Cannot open %s\n",t);
  fclose (f);
  return 2;
 }
 for (i=0;i<512;++i)
 {
  if (!WriteSector(f,i))
  {
   printf ("Error writing sector %u\n",i);
   return 4;
  }
 }
 fclose (f);
 return 0;
}
